import type { SynchronousGroupCache, SynchronousCache } from './types/SyncDataSources';
import type { Cache, GroupCache } from './types/DataSources';
import type { Logger } from './util/Logger';
import type { InMemoryGroupCacheConfiguration } from './memory/InMemoryGroupCache';
import type { NotificationPublisher } from './notifications/NotificationPublisher';
import type { GroupNotificationPublisher } from './notifications/GroupNotificationPublisher';
import type { AbstractNotificationConsumer } from './notifications/AbstractNotificationConsumer';
import type { InMemoryCacheConfiguration } from './memory/InMemoryCache';
export type LoaderErrorHandler = (err: Error, key: string | undefined, loader: Record<string, any>, logger: Logger) => void;
export declare const DEFAULT_LOAD_ERROR_HANDLER: LoaderErrorHandler;
export declare const DEFAULT_CACHE_ERROR_HANDLER: LoaderErrorHandler;
export type CommonCacheConfig<LoadedValue, CacheType extends Cache<LoadedValue> | GroupCache<LoadedValue> = Cache<LoadedValue>, InMemoryCacheConfigType extends InMemoryCacheConfiguration | InMemoryGroupCacheConfiguration = InMemoryCacheConfiguration, InMemoryCacheType extends SynchronousCache<LoadedValue> | SynchronousGroupCache<LoadedValue> = SynchronousCache<LoadedValue>, NotificationPublisherType extends NotificationPublisher<LoadedValue> | GroupNotificationPublisher<LoadedValue> = NotificationPublisher<LoadedValue>> = {
    logger?: Logger;
    cacheUpdateErrorHandler?: LoaderErrorHandler;
    loadErrorHandler?: LoaderErrorHandler;
    inMemoryCache?: InMemoryCacheConfigType | false;
    asyncCache?: CacheType;
    notificationConsumer?: AbstractNotificationConsumer<LoadedValue, InMemoryCacheType>;
    notificationPublisher?: NotificationPublisherType;
};
export declare abstract class AbstractCache<LoadedValue, LoadChildType = Promise<LoadedValue | undefined | null> | undefined, CacheType extends Cache<LoadedValue> | GroupCache<LoadedValue> = Cache<LoadedValue>, InMemoryCacheType extends SynchronousCache<LoadedValue> | SynchronousGroupCache<LoadedValue> = SynchronousCache<LoadedValue>, InMemoryCacheConfigType extends InMemoryCacheConfiguration | InMemoryGroupCacheConfiguration = InMemoryCacheConfiguration, NotificationPublisherType extends NotificationPublisher<LoadedValue> | GroupNotificationPublisher<LoadedValue> = NotificationPublisher<LoadedValue>> {
    protected readonly inMemoryCache: InMemoryCacheType;
    protected readonly asyncCache?: CacheType;
    protected readonly logger: Logger;
    protected readonly cacheUpdateErrorHandler: LoaderErrorHandler;
    protected readonly loadErrorHandler: LoaderErrorHandler;
    protected readonly runningLoads: Map<string, LoadChildType>;
    private readonly notificationConsumer?;
    protected readonly notificationPublisher?: NotificationPublisherType;
    abstract isGroupCache(): boolean;
    private initPromises;
    constructor(config: CommonCacheConfig<LoadedValue, CacheType, InMemoryCacheConfigType, InMemoryCacheType, NotificationPublisherType>);
    init(): Promise<void>;
    invalidateCache(): Promise<void>;
    close(): Promise<void>;
}
