"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisGroupNotificationConsumer = void 0;
const AbstractNotificationConsumer_1 = require("../notifications/AbstractNotificationConsumer");
class RedisGroupNotificationConsumer extends AbstractNotificationConsumer_1.AbstractNotificationConsumer {
    redis;
    channel;
    constructor(redis, config) {
        super(config.serverUuid);
        this.redis = redis;
        this.channel = config.channel;
    }
    async close() {
        await this.redis.unsubscribe(this.channel);
    }
    subscribe() {
        return this.redis.subscribe(this.channel).then(() => {
            this.redis.on('message', (channel, message) => {
                const parsedMessage = JSON.parse(message);
                // this is a local message, ignore
                if (parsedMessage.originUuid === this.serverUuid) {
                    return;
                }
                if (parsedMessage.actionId === 'DELETE_FROM_GROUP') {
                    return this.targetCache.deleteFromGroup(parsedMessage.key, parsedMessage.group);
                }
                if (parsedMessage.actionId === 'DELETE_GROUP') {
                    return this.targetCache.deleteGroup(parsedMessage.group);
                }
                if (parsedMessage.actionId === 'CLEAR') {
                    return this.targetCache.clear();
                }
            });
        });
    }
}
exports.RedisGroupNotificationConsumer = RedisGroupNotificationConsumer;
//# sourceMappingURL=RedisGroupNotificationConsumer.js.map