"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNotificationPair = void 0;
const RedisNotificationConsumer_1 = require("./RedisNotificationConsumer");
const RedisNotificationPublisher_1 = require("./RedisNotificationPublisher");
const node_crypto_1 = require("node:crypto");
function createNotificationPair(config) {
    const serverUuid = (0, node_crypto_1.randomUUID)();
    if (config.publisherRedis === config.consumerRedis) {
        throw new Error('Same Redis client instance cannot be used both for publisher and for consumer, please create a separate connection');
    }
    const consumer = new RedisNotificationConsumer_1.RedisNotificationConsumer(config.consumerRedis, {
        channel: config.channel,
        serverUuid,
    });
    const publisher = new RedisNotificationPublisher_1.RedisNotificationPublisher(config.publisherRedis, {
        channel: config.channel,
        errorHandler: config.errorHandler,
        serverUuid,
    });
    return {
        publisher,
        consumer,
    };
}
exports.createNotificationPair = createNotificationPair;
//# sourceMappingURL=RedisNotificationFactory.js.map